// maf2tba.c -- convert a pairwise maf file to a tba file

#include "util.h"
#include "seq.h"
#include "maf.h"

void maf_make1(struct mafAli *a, unsigned char *s, int start, int size,
  char *src, int srcSize) {
	int i;
	struct mafComp *c = a->components;

	a->textSize = size;
	a->score = 0.0;
	a->next = NULL;
	c->start = start;
	c->size = size;
	c->src = copy_string(src);
	c->srcSize = srcSize;
	c->strand = '+';
	c->text = ckalloc((size+1)*sizeof(char));
	for (i = 0; i < size; ++i)
		c->text[i] = s[i+start];
	c->text[i] = '\0';
	mafWrite(stdout, a);
	free(c->text);
	free(c->src);
}

int main(int argc, char **argv) {
	struct mafFile *mf;
	struct mafAli A, *a, *b;
	struct mafComp C, *c;
	SEQ *sf1, *sf2;
	unsigned char *s1, *s2;
	int end1, end2, size;
	
	if (argc != 4)
		fatal("args: maf-file species1 species2");
	a = &A;
	c = &C;
	a->components = c;
	c->next = NULL;

	mf = mafOpen(argv[1],1);
	sf1 = seq_get(argv[2]);
	s1 = SEQ_CHARS(sf1);
	sf2 = seq_get(argv[3]);
	s2 = SEQ_CHARS(sf2);
	mafWriteStart(stdout, "maf2tba");
	end1 = end2 = 0;
	while ((b = mafNext(mf)) != NULL) {
		c = b->components;
		if ((size = c->start - end1) > 0)
			maf_make1(a, s1, end1, size, argv[2], SEQ_LEN(sf1));
		else if (size < 0)
			fatalf("%s out of order in alignment to %s around %d",
			  argv[2], argv[3], c->start);
		end1 = c->start + c->size;
		c = c->next;
		if ((size = c->start - end2) > 0)
			maf_make1(a, s2, end2, size, argv[3], SEQ_LEN(sf2));
		else if (size < 0)
			fatalf("%s out of order in alignment to %s around %d",
			  argv[3], argv[2], c->start);
		end2 = c->start + c->size;
		mafWrite(stdout, b);
		mafAliFree(&b);
	}
	if ((size = SEQ_LEN(sf1) - end1) > 0)
		maf_make1(a, s1, end1, size, argv[2], SEQ_LEN(sf1));
	if ((size = SEQ_LEN(sf2) - end2) > 0)
		maf_make1(a, s2, end2, size, argv[3], SEQ_LEN(sf2));
	return 0;
}
